// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"
#include "scene/components/display/MeshRenderer.hpp"
#include "scene/components/display/MeshHost.hpp"
#include "display/mesh/QuadMesh.hpp"
#include "display/Tileset.hpp"

#define TILED_SPRITE_FLIP_Y FLAG_DEFINE(0)
#define TILED_SPRITE_FLIP_X FLAG_DEFINE(1)

namespace Dawn {
  class TiledSprite : public SceneItemComponent {
    protected:
      MeshHost *host = nullptr;
      Tileset *tileset = nullptr;
      flag_t flipState = TILED_SPRITE_FLIP_Y;
      int32_t tileIndex = -1;
      glm::vec2 xy0 = glm::vec2(0, 0);
      glm::vec2 xy1 = glm::vec2(1, 1);

      glm::vec2 getUV0();
      glm::vec2 getUV1();

    public:
      TiledSprite(SceneItem *item);

      std::vector<SceneItemComponent*> getDependencies() override;
      void onStart() override;

      /**
       * Sets which tileset to use for this sprite.
       * 
       * @param tileset Tileset to use.
       */
      void setTileset(Tileset *tileset);

      /**
       * Sets the tileset for the sprite, and autosizes the sprite based on
       * this tileset.
       * 
       * @param gridTileset Tileset to use.
       * @param center The center offset of the sprite.
       */
      void setTilesetAndSize(TilesetGrid *gridTileset, glm::vec2 center);

      /**
       * Sets the tileset for the sprite, and autosizes the sprite based on
       * this tileset. This will put the sprite centered on its origin.
       * 
       * @param gridTileset Tileset to use.
       */
      void setTilesetAndSize(TilesetGrid *gridTileset);

      /**
       * Updates the selected tile.
       * 
       * @param tile Tile to use.
       */
      void setTile(int32_t tile);

      /**
       * Adjust how the sprite is flippxed.
       * 
       * @param flippedState Flipped axis flags.
       */
      void setFlippedState(flag_t flippedState);

      /**
       * Sets the dimensions of this tiled sprite.
       * 
       * @param size Size of the sprite.
       * @param center Negative center offset.
       */
      void setSize(glm::vec2 size, glm::vec2 center);

      /**
       * Sets the size of this sprite. This will center the sprite on its origin
       * 
       * @param size Size of the sprite.
       */
      void setSize(glm::vec2 size);
  };
}