// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"
#include "display/RenderTarget.hpp"
#include "scene/components/display/Camera.hpp"

namespace Dawn {
  enum UIDrawType {
    UI_DRAW_TYPE_WORLD_ABSOLUTE,
    UI_DRAW_TYPE_WORLD_CAMERA_RELATIVE,
    UI_DRAW_TYPE_CAMERA_OVERLAY
  };

  class UICanvas : public SceneItemComponent {
    public:
      /**
       * Creates a UI Canvas Scene Item Element, and attaches it to the provided
       * scene.
       * 
       * @param scene Scene to create the UI Canvas for.
       * @return Created UI Canvas.
       */
      static UICanvas * create(Scene *scene);

      //======================================================================//

      enum UIDrawType drawType = UI_DRAW_TYPE_WORLD_CAMERA_RELATIVE;

      /**
       * Constructs the UI Canvas Scene Item Component.
       * 
       * @param item Item that this canvas item belongs to.
       */
      UICanvas(SceneItem *item);

      void onStart() override;
      void onDispose() override;
  };
}